/** 
 * @file DemoRecognizer.h
 *
 * This file contains the clsDemoRecognizer class, the wrapper of LVSpeechPort object.
 */

#ifndef DemoRecognizer_h
#define DemoRecognizer_h

#include <LVSpeechPort.h>
#include <string>


/**
 * @brief the clsDemoRecognizer class.
 * 
 *  This is the holder of LVSpeechPort object. In this demo,
 *  the DemoRecognizer will do regular decode on voice channel 1,
 *  and use builtin boolean grammar to confirm regular decode result on 
 *  voice channel 2. 
 */
class clsDemoRecognizer
{
public:
	/**
	 *  @brief The constructor
	 */
	clsDemoRecognizer();

	/**
	 * @brief The destructor
     */
	~clsDemoRecognizer();

	/**
	 * @brief The Initialization function.
	 *  
 	 * @return int
     *		\li Description: 0 indicates success, 1 indicates failure.
     *		\li Possible Value(s): 0 or -1.
     */
	int Initialize();

	/**
	 *  @brief Logging callback function.
	 * 
	 *  @param [in] msg: const char* 
	 * 		\li Description: log message. 
	 *  @param [in] userdata: void*
	 * 		\li Description: Callback function user data pointer.
 	 *  @return void
     */
	static void Log(const char* msg, void* userdata);

	/**
	 *  @brief Do recognition on given grammar.
	 * 
	 *  @param [in] audio: const char* 
	 *		\li Description: Audio buffer.
	 *  @param [in] audio_length: unsigned int 
	 *		\li Description: Audio buffer size in byte.
	 *	@param [in] format: SOUND_FORMAT
	 *		\li Description: Audio format.
	 *		\li Acceptable Value(s): ULAW_8KHZ,PCM_8KHZ,PCM_16KHZ,ALAW_8KHZ,
	 *  @param [in] grammar_uri: const std::string & 
	 *		\li Description: grammar URI.
 	 *  @return int
     *  \li Description: 0 indicates success, and -1 indicates failure.
     *  \li Possible Value(s): 0 or -1.
     */
	int Recognize(const char* audio, unsigned int audio_length, SOUND_FORMAT format, const std::string & grammar_uri);


	/**
	 * @brief Retrieve number of interpretations.
	 * 
	 * This function calls the LVSRE API to retrieve number of interpretations upon successful decode.
	 *
 	 *  @return int
     *  \li Description: Number of distinct interpretations. 
     */
	int GetNumberOfInterpretations();

	/**
	 * @brief Retrieve interpretation object.
	 * 
	 * After a successful decode, there are one or more interpretations.This function returns the 
	 * interpretation indexed by index.
	 * 
	 *  @param [in] index: unsigned int 
	 *		\li Description:The index of the interpretation you want to retrieve.
	 *		\li Acceptable Value(s): 0 ~ "Number of interpretations - 1"
 	 *  @return LVInterpretation
     *		\li Description: LVInterpretation object. If the index is invalid, an empty object will be returned.
     */
	LVInterpretation GetInterpretation(unsigned int index);

	/**
	 * @brief Do decode on boolean grammar to confirm previous decode result.
	 * 
	 *  @param [in] audio: const char* 
	 *		\li Description: Audio buffer.
	 *  @param [in] audio_length: unsigned int 
	 *		\li Description: Audio buffer size in byte.
	 *	@param [in] format: SOUND_FORMAT
	 *		\li Description: Audio format.
	 *		\li Acceptable Value(s): ULAW_8KHZ,PCM_8KHZ,PCM_16KHZ,ALAW_8KHZ,
 	 *  @return int
     *  \li Description: 0 indicates success, and -1 indicates failure.
     *  \li Possible Value(s): 0 or -1.
     */
	int Confirm(const char* audio, unsigned int audio_length, SOUND_FORMAT format);

	/**
	 * @brief Retrieve number of interpretations on confirmation decode.
	 * 
	 * This function calls the LVSRE API to retrieve number of interpretations upon successful decode.
	 *
 	 *  @return int
     *  \li Description: Number of distinct interpretations. 
     */
	int GetNumberOfConfirmInterpretation();

	/**
	 * @brief Retrieve interpretation object on confirmation decode.
	 * 
	 * We know that the builtin grammar is not ambiguous. So the confirmation decode returns either one or 
	 * zero interpretation. So we always use index 0 here.
	 * 
 	 *  @return LVInterpretation
     *		\li Description: LVInterpretation object.
     */
	LVInterpretation GetConfirmInterpretation();

protected:
	LVSpeechPort Port;/**< LVSpeechPort object.*/

};

#endif //DemoRecognizer_h
