/** 
 * @file LVSpeechPortConsole.cpp
 *
 * This file contains main().
 */

#include "DemoRecognizer.h"
#include "audio.h"
#include <string>
#include <iostream>
#include <conio.h>
#include <windows.h>
#include <fstream>

/**
 * \mainpage LVSpeechPortConsoleDemo Main Page.
 * 
 * \section intro_sec Introduction
 * 
 * The purpose of this console application is to demonstrate the basic usage of LumenVox 
 * Speech Recognition Engine (LVSRE) API. For detail information about the API, please
 * refer to the API documentation. This application also demonstrates the usage of W3C
 * SRGS grammars and Semantic Interpretation tags with LVSRE API. 
 *
 * \section build_usage Build And Usage.
 *
 * \subsection build Build
 * This project was tested and compiled on Microsoft Windows 2000 Professional Edition,
 * Microsoft Platform SDK November 2001 Edition, LumenVox SRE SDK 5.0, Microsoft Visual 
 * C++ 6.0 Enterprise Edition.
 * 
 * - Install LumenVox SRE. This will install the recognition engine and SDK, and set all environment variables.
 * - Build this project with VC++ 6.0. After build, three test grammars will be copied to $LVBIN directory. ($LVBIN is the directory where the SRE is installed.)
 *
 * \subsection usage Usage
 * Start the SRE service, then start this demo, you will be asked to enter the URI of the grammar that you want 
 * to test with. Project provides three pre-defined grammars, "names.gram", "states.gram" and 
 * "spanish.gram". You can also create you own W3C SRGS grammars.
 * 
 * Once the grammar is successfully loaded. You can see the content of the grammar on the console. Then
 * you may press any key to record your utterance with the microphone. The recording stops
 * automatically when you finish talking. The recorded audio will be sent to the engine for 
 * recognition. 
 *
 * The recognized result, including input text, confidence score and interpretation, 
 * will be displayed. Then you can press any key and say "yes" or "no" to confirm.
 * This utterance will be recognized using builtin
 * boolean grammar.
 *
 * Once the confirmation is done. The demo goes back to main menu. You can try another 
 * decode or exit.
 *
 * \section filelist File List
 * 
 * - names.gram: Names demo grammar. The grammar contains following names:
 *   -# John Wilson
 *   -# Laura Gerking
 *   -# Jeb Burnside
 *   -# Don Brown
 *   -# Bob Brown
 *   -# Brenda Carol
 *   -# Howard Fried
 *   -# Max
 * - states.gram: State names demo grammar.The grammar contains all state names in the United States.
 * - spanish.gram: Spanish phrases demo grammar. The grammar contains following phrases:
 *   -# credito
 *   -# si
 *   -# el bao
 *   -# hola
 *
 * - DemoRecognizer.h/.cpp: The recognizer class, a wrapper of the LVSpeechPort class.
 * - Audio.h/.cpp: The recording routines. 
 * - LVSpeechPortConsole.cpp: Entry point of this application.
 * 
 * \section modifications
 *
 * - 6/18/08:
 *   states.gram modified to workaround an engine problem where lack of trailing
 *   CR/LF at end of file caused problems.
 *
 *   Modified to use _cgets_s in place of _cgets, which appears to have a problem
 *   in the VS 2005 implementation.
 *
 *   Changed calls to GetAudio to include an additional RecordingStatus parameter 
 *   that is returned to the caller.
 *
 *   Additional check for valid audio format added.
 *
 */

SOUND_FORMAT format;
std::string last_grammar_uri;


	/**
	 * @brief Print main menu and retrieve user's choice.
	 * 
 	 *  @return int
     */
int DoMainMenu();

	/**
	 * @brief The DoRecognition function
	 * 
	 *  @param [in] rec: clsDemoRecognizer & 
	 *		\li Description:reference to the recognizer object.
 	 *  @return int
     *  \li Description: n/a.
     *  \li Possible Value(s): n/a.
     */
int DoRecognition(clsDemoRecognizer & rec);


///////////////////////////////////////////////////////////////
/**
 *  @brief The main function
 * 
 *  @param [in] argc: int 
 *		\li Description: This executable takes one parameter, the path of the directory containing the grammars.
 *		\li Default Value:n/a 
 *		\li Acceptable Value(s):n/a
 *  @param [in] argv[]: char* 
 *		\li Description:n/a 
 *		\li Default Value:n/a 
 *		\li Acceptable Value(s):n/a
 *  @return int
 */
int main(int argc, char* argv[])
{

	/////////////////////////////////////////////////
	//Change windows console code page, so that 
	//the accent characters are displayed properly.
	/////////////////////////////////////////////////
	SetConsoleOutputCP(1250);

	/////////////////////////////////////////////////
	//Initialize default grammar url
	//Set default grammar url, the state.gram in the demo executable directory.
	//////////////////////////////////////////////////
	//char Buffer[1024];
	last_grammar_uri = "C:/Program Files/Lumenvox/Engine/Examples/LVSpeechPortConsole/states.gram";

	//////////////////////////////////////////////////
	//Create a recognizer object
	//////////////////////////////////////////////////
	clsDemoRecognizer recognizer;

	char cMChoice = 'A';

	//////////////////////////////////////////////////
	//make sure the current sound format is valid
	//////////////////////////////////////////////////
	format = GetSoundFormat();
	if (format == UNK_FORMAT)
		return 0;

	//////////////////////////////////////////////////
	//Initialize recognizer.
	//////////////////////////////////////////////////
	if (recognizer.Initialize() == -1)
	{
		return -1;
	}

	while(cMChoice != '2')
	{
		while(!(cMChoice = DoMainMenu()))
			std::cout << "\n\nError:        Invalid Entry         \n\n";

		switch(cMChoice)
		{
		case '1':
			if(DoRecognition(recognizer))
				return 0;
			std::cout << "Going back to main menu." << std::endl;

			break;
		}
	}

	return 0;
}


///////////////////////////////////////////////////////////////
int DoMainMenu()
{
	char cKeyPress;

	std::cout << "\n\n\n\n\n********************************************\n";
	std::cout << "               MAIN MENU                    \n";
	std::cout << "********************************************\n";


	std::cout << "Select desired option.\n\n";

	std::cout << "1. Do Recognition\n";
	std::cout << "2. Exit\n\n";

	std::cout << "/>:";

	while(!_kbhit()) Sleep(50);

	switch(cKeyPress = getche())
	{
	case '1':
	case '2':
		return cKeyPress;
		break;
	}

	return 0;
}

///////////////////////////////////////////////////////////////
int DoRecognition(clsDemoRecognizer & rec)
{

	//////////////////////////////////////////////////////////
	//Get grammar uri
	//////////////////////////////////////////////////////////
	std::cout << "\nEnter the grammar uri [default(depends on your installation location):" << last_grammar_uri << "]\n:";
	std::string this_grammar_uri;
	//////////////////////////////////////////////////////////
	//If the uri entered by user is empty, then use default
	//////////////////////////////////////////////////////////
	char input[255] = {(char)0};

	size_t BytesRead;

	_cgets_s(input, sizeof(input), &BytesRead);
	this_grammar_uri = input;
	if (this_grammar_uri.find_first_not_of(" \t") == std::string::npos)
		this_grammar_uri = last_grammar_uri;


	/////////////////////////////////////////////////////////
	//Display the grammar
	/////////////////////////////////////////////////////////
	std::ifstream grammar(this_grammar_uri.c_str());
	if (!grammar)
	{
		std::cout << "Unable to open grammar " << this_grammar_uri << std::endl;
		return 0;
	}
	std::cout << "**************** Grammar Content ****************" << std::endl;
	std::string line;
	while(std::getline(grammar, line))
		std::cout << line << std::endl;
	grammar.close();
	std::cout << "************** Grammar Content End **************" << std::endl;
	std::cout << std::endl;

	/////////////////////////////////////////////////////////
	//Start recording
	/////////////////////////////////////////////////////////
	std::cout << "\nPress any key and say a sentence defined by the grammar.\n";

	int retv;
	const char* pBuffer = NULL;
	int iBuffSize;
	/////////////////////////////////////////////////////////
	//record audio
	/////////////////////////////////////////////////////////
	while(!_kbhit()) Sleep(50);
	while(!getch());

	if (StartRecord())
		return 0;

	bool RecordingStatus = true;

	while(RecordingStatus == true)
	{
		// See if recording is finished. If so, get a pointer to the audio data
		pBuffer = GetAudio(&iBuffSize, &RecordingStatus);
		Sleep(100);
	}

	if (iBuffSize)
	{
		/////////////////////////////////////////////////////////
		//do decode.
		/////////////////////////////////////////////////////////
		retv = rec.Recognize(pBuffer, iBuffSize, format, this_grammar_uri);
	}
	else
	{
		return -1;
	}

	if (retv == -1)
		return 0;
	/////////////////////////////////////////////////////////
	//Get number of interpretation.
	/////////////////////////////////////////////////////////
	int num_of_interpretation;
	num_of_interpretation = rec.GetNumberOfInterpretations();
	if (num_of_interpretation <= 0)
	{
		std::cout << "Recognition Failed." << std::endl;
		return 0;
	}
	else
	{
		std::cout << "Number of interpretation = " << num_of_interpretation << std::endl;
	}

	last_grammar_uri = this_grammar_uri;

	int i;
	for (i=0; i<num_of_interpretation; i++)
	{
		/////////////////////////////////////////////////////////
		//Interpretation object contains semantic data object and other info, 
		//such as confidence, input text ...
		/////////////////////////////////////////////////////////
		LVInterpretation inter = rec.GetInterpretation(i);
		/////////////////////////////////////////////////////////
		//Dispaly input text, the recognized text
		/////////////////////////////////////////////////////////
		std::cout << "*** Input Text ***" <<std::endl;
		std::cout << inter.InputSentence() << std::endl;
		std::cout << "*** Confidence ***" <<std::endl;
		std::cout << inter.Score() << std::endl;
		std::cout << "*** Semantic Interpretation ***" << std::endl;
		LVSemanticData data = inter.ResultData( );
		/////////////////////////////////////////////////////////
		//Use this function to retrieve the XML or ECMAScript representation of a semantic object.
		/////////////////////////////////////////////////////////
		std::cout << data.Print(SI_FORMAT_XML) << std::endl;

		std::cout << "Press any key and say \"yes\" or \"no\" to confirm the result(yes/no)" << std::endl;
		bool done = false;
		bool confirmed = false;
		while (!done)
		{
			/////////////////////////////////////////////////////////
			//record confirmation audio
			/////////////////////////////////////////////////////////
			while(!_kbhit()) Sleep(50);
			while(!getch());
			StartRecord();

			bool RecordingStatus = true;
			while(RecordingStatus == true)
			{
				// See if recording is finished. If so, get a pointer to the audio data
				pBuffer = GetAudio(&iBuffSize, &RecordingStatus);
				Sleep(100);
			}

			/////////////////////////////////////////////////////////
			//Decode using confirmation (yes/no) grammar.
			//Note, here we are using a different Voice Channel, so that we won't 
			//wipe out previous decode results.
			/////////////////////////////////////////////////////////
			retv = rec.Confirm(pBuffer, iBuffSize, format);
			int num_of_inter_confirm = rec.GetNumberOfConfirmInterpretation();
			
			if (num_of_inter_confirm == 1)
			{
				LVSemanticData confirmdata = rec.GetConfirmInterpretation().ResultData();
				/////////////////////////////////////////////////////////
				//Here, we know that the semantic data is a string type, 
				//so we use GetString() funtion to retrieve the pointer.
				/////////////////////////////////////////////////////////
				if(!stricmp(confirmdata.GetString(), "true"))
				{
					std::cout << "\nRecognition Confirmed\n";
					Sleep(1500);
					done = true;
					confirmed = true;
				
				}
				else if (!stricmp(confirmdata.GetString(), "false"))
				{
					std::cout << "\nRecognition Denied\n\n";		
					done = true;
				}
				else
				{
					std::cout << "I did not understand if you said yes or no.";
					std::cout << "Press any key and say \"yes\" or \"no\" to confirm the result(yes/no)" << std::endl;
				}
			}
		}
		if (confirmed) break;
	}

	return 0;
}
